@@ -542,6 +542,7 @@ int dowicked(int rnum){
 
 /* perform misc command */
 int domisc(void){
+  CBDATUM *odatum, *ndatum;
   CBLIST *olist, *nlist, *elems, *glist;
   CBMAP *omap, *nmap, *pairs, *gmap;
   int i, j, ssiz, osiz, tsiz, jl;
@@ -549,6 +550,30 @@ int domisc(void){
   const char *op, *np;
   time_t t;
   printfflush("<Miscellaneous Test>\n\n");
+  printfflush("Checking basic datum ... ");
+  odatum = cbdatumopen("x", -1);
+  for(i = 0; i < 1000; i++){
+    cbdatumcat(odatum, "x", 1);
+  }
+  cbdatumclose(odatum);
+  tmp = cbmalloc(3);
+  memcpy(tmp, "abc", 3);
+  odatum = cbdatumopenbuf(tmp, 3);
+  for(i = 0; i < 1000; i++){
+    cbdatumcat(odatum, ".", 1);
+  }
+  ndatum = cbdatumdup(odatum);
+  for(i = 0; i < 1000; i++){
+    cbdatumcat(ndatum, "*", 1);
+  }
+  for(i = 0; i < 1000; i++){
+    tmp = cbmalloc(3);
+    memcpy(tmp, "123", 3);
+    cbdatumsetbuf(ndatum, tmp, 3);
+  }
+  cbdatumclose(ndatum);
+  cbdatumclose(odatum);
+  printfflush("ok\n");
   printfflush("Checking serialization of list ... ");
   olist = cblistopen();
   for(i = 1; i <= 1000; i++){
