@@ -27,6 +27,7 @@
 #define DP_BNUMOFF     32                /* offset of the region for the bucket number */
 #define DP_RNUMOFF     40                /* offset of the region for the record number */
 #define DP_DEFBNUM     8191              /* default bucket number */
+#define DP_FBPOOLSIZ   16                /* size of free block pool */
 #define DP_COALMAX     3                 /* number of trials for record coalescence */
 #define DP_ENTBUFSIZ   128               /* size of the entity buffer */
 #define DP_STKBUFSIZ   256               /* size of the stack key buffer */
@@ -126,8 +127,8 @@ const char *dperrmsg(int ecode){
 
 /* Get a database handle. */
 DEPOT *dpopen(const char *name, int omode, int bnum){
-  int mode, fd, inode, mtime, fsiz, rnum, msiz;
-  char hbuf[DP_HEADSIZ], *map, c;
+  char hbuf[DP_HEADSIZ], *map, c, *tname;
+  int i, mode, fd, inode, mtime, fsiz, rnum, msiz, *fbpool;
   struct stat sbuf;
   DEPOT *depot;
   assert(name);
@@ -153,7 +154,8 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
       return NULL;
     }
   }
-  if(fstat(fd, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)){
+  if(fstat(fd, &sbuf) == -1 || !S_ISREG(sbuf.st_mode) ||
+     (sbuf.st_ino == 0 && lstat(name, &sbuf) == -1)){
     close(fd);
     dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return NULL;
@@ -227,13 +229,19 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     dpecodeset(DP_EMAP, __FILE__, __LINE__);
     return NULL;
   }
-  if(!(depot = malloc(sizeof(DEPOT))) || !(depot->name = dpstrdup(name))){
-    munmap(map, msiz);
+  tname = NULL;
+  fbpool = NULL;
+  if(!(depot = malloc(sizeof(DEPOT))) || !(tname = dpstrdup(name)) ||
+     !(fbpool = malloc(DP_FBPOOLSIZ * sizeof(int) * 2))){
+    free(fbpool);
+    free(tname);
     free(depot);
+    munmap(map, msiz);
     close(fd);
     dpecodeset(DP_EALLOC, __FILE__, __LINE__);
     return NULL;
   }
+  depot->name = tname;
   depot->wmode = (mode & DP_OWRITER);
   depot->inode = inode;
   depot->mtime = mtime;
@@ -246,8 +254,11 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
   depot->rnum = rnum;
   depot->fatal = FALSE;
   depot->ioff = 0;
-  depot->mroff = -1;
-  depot->mrsiz = -1;
+  depot->fbpool = fbpool;
+  for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+    depot->fbpool[i] = -1;
+    depot->fbpool[i+1] = -1;
+  }
   depot->align = 0;
   return depot;
 }
@@ -273,6 +284,7 @@ int dpclose(DEPOT *depot){
     err = TRUE;
     dpecodeset(DP_ECLOSE, __FILE__, __LINE__);
   }
+  free(depot->fbpool);
   free(depot->name);
   free(depot);
   if(fatal){
@@ -286,7 +298,7 @@ int dpclose(DEPOT *depot){
 /* Store a record. */
 int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int dmode){
   int head[DP_RHNUM], next[DP_RHNUM];
-  int i, hash, bi, off, entoff, ee, newoff, rsiz, nsiz, fdel, mroff, mrsiz;
+  int i, j, hash, bi, off, entoff, ee, newoff, rsiz, nsiz, fdel, mroff, mrsiz, mi, min;
   char ebuf[DP_ENTBUFSIZ], *tval, *swap;
   assert(depot && kbuf && vbuf);
   if(depot->fatal){
@@ -326,9 +338,12 @@ int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
       }
     } else {
       for(i = 0; i < DP_COALMAX && nsiz > rsiz && off + rsiz < depot->fsiz; i++){
-        if(off + rsiz == depot->mroff){
-          depot->mroff = -1;
-          depot->mrsiz = -1;
+        for(j = 0; j < DP_FBPOOLSIZ * 2; j += 2){
+          if(depot->fbpool[j] == off + rsiz){
+            depot->fbpool[j] = -1;
+            depot->fbpool[j+1] = -1;
+            break;
+          }
         }
         if(!dprechead(depot, off + rsiz, next, NULL, NULL)) return FALSE;
         if(!(next[DP_RHIFLAGS] & DP_RECFREUSE)) break;
@@ -368,8 +383,24 @@ int dpput(DEPOT *depot, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
         vsiz += head[DP_RHIVSIZ];
         vbuf = tval;
       }
-      mroff = depot->mroff;
-      mrsiz = depot->mrsiz;
+      mi = -1;
+      min = -1;
+      for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+        if(depot->fbpool[i+1] < nsiz) continue;
+        if(mi == -1 || depot->fbpool[i+1] < min){
+          mi = i;
+          min = depot->fbpool[i+1];
+        }
+      }
+      if(mi >= 0){
+        mroff = depot->fbpool[mi];
+        mrsiz = depot->fbpool[mi+1];
+        depot->fbpool[mi] = -1;
+        depot->fbpool[mi+1] = -1;
+      } else {
+        mroff = -1;
+        mrsiz = -1;
+      }
       if(!dprecdelete(depot, off, head, TRUE)){
         free(tval);
         depot->fatal = TRUE;
@@ -794,8 +825,10 @@ int dpoptimize(DEPOT *depot, int bnum){
   depot->fsiz = tdepot->fsiz;
   depot->bnum = tdepot->bnum;
   depot->ioff = 0;
-  depot->mroff = -1;
-  depot->mrsiz = -1;
+  for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+    depot->fbpool[i] = -1;
+    depot->fbpool[i+1] = -1;
+  }
   depot->msiz = tdepot->msiz;
   depot->map = mmap(0, depot->msiz, PROT_READ | PROT_WRITE, MAP_SHARED, depot->fd, 0);
   if(depot->map == MAP_FAILED){
@@ -933,7 +966,7 @@ int dpremove(const char *name){
   struct stat sbuf;
   DEPOT *depot;
   assert(name);
-  if(stat(name, &sbuf) == -1){
+  if(lstat(name, &sbuf) == -1){
     dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return FALSE;
   }
@@ -953,7 +986,7 @@ int dprepair(const char *name){
   int fd, fsiz, flags, bnum, tbnum, err, head[DP_RHNUM], off, rsiz, ksiz, vsiz;
   struct stat sbuf;
   assert(name);
-  if(stat(name, &sbuf) == -1){
+  if(lstat(name, &sbuf) == -1){
     dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return FALSE;
   }
@@ -1872,9 +1905,15 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
    `cat' specifies whether it is concatenate mode or not.
    The return value is true if successful, or, false on failure. */
 static int dprecover(DEPOT *depot, int off, int *head, const char *vbuf, int vsiz, int cat){
-  int hsiz, hoff, voff;
+  int i, hsiz, hoff, voff;
   assert(depot && off >= 0 && head && vbuf && vsiz >= 0);
-  if(depot->mroff == off) depot->mroff = -1;
+  for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+    if(depot->fbpool[i] == off){
+      depot->fbpool[i] = -1;
+      depot->fbpool[i+1] = -1;
+      break;
+    }
+  }
   if(cat){
     head[DP_RHIFLAGS] = 0;
     head[DP_RHIPSIZ] -= vsiz;
@@ -1903,10 +1942,27 @@ static int dprecover(DEPOT *depot, int off, int *head, const char *vbuf, int vsi
    `reusable' specifies whether the region is reusable or not.
    The return value is true if successful, or, false on failure. */
 static int dprecdelete(DEPOT *depot, int off, int *head, int reusable){
+  int i, mi, min;
   assert(depot && off >= 0 && head);
   if(reusable){
-    depot->mroff = off;
-    depot->mrsiz = dprecsize(head);
+    mi = -1;
+    min = -1;
+    for(i = 0; i < DP_FBPOOLSIZ * 2; i += 2){
+      if(depot->fbpool[i] == -1){
+        depot->fbpool[i] = off;
+        depot->fbpool[i+1] = dprecsize(head);
+        mi = -1;
+        break;
+      }
+      if(min < 0 || depot->fbpool[i+1] < min){
+        mi = i;
+        min = depot->fbpool[i+1];
+      }
+    }
+    if(mi >= 0){
+      depot->fbpool[mi] = off;
+      depot->fbpool[mi+1] = dprecsize(head);
+    }
   }
   return dpseekwritenum(depot->fd, off + DP_RHIFLAGS * sizeof(int),
                         DP_RECFDEL | (reusable ? DP_RECFREUSE : 0));
