@@ -63,7 +63,7 @@ int doput(const char *name, const char *kbuf, int ksiz, const char *vbuf, int vs
           int dmode, int align);
 int doout(const char *name, const char *kbuf, int ksiz);
 int doget(const char *name, const char *kbuf, int ksiz, int start, int max, int ox, int nb);
-int dolist(const char *name, int ox);
+int dolist(const char *name, int kb, int vb, int ox);
 int dooptimize(const char *name, int bnum, int align);
 int doinform(const char *name);
 int doremove(const char *name);
@@ -126,7 +126,7 @@ void usage(void){
   fprintf(stderr, "  %s out [-kx|-ki] name key\n", progname);
   fprintf(stderr, "  %s get [-kx|-ki] [-start num] [-max num] [-ox] [-n] name key\n",
           progname);
-  fprintf(stderr, "  %s list [-ox] name\n", progname);
+  fprintf(stderr, "  %s list [-k|-v] [-ox] name\n", progname);
   fprintf(stderr, "  %s optimize [-bnum num] [-na] name\n", progname);
   fprintf(stderr, "  %s inform name\n", progname);
   fprintf(stderr, "  %s remove name\n", progname);
@@ -390,12 +390,18 @@ int runget(int argc, char **argv){
 /* parse arguments of list command */
 int runlist(int argc, char **argv){
   char *name;
-  int i, ox, rv;
+  int i, kb, vb, ox, rv;
   name = NULL;
+  kb = FALSE;
+  vb = FALSE;
   ox = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-ox")){
+      if(!strcmp(argv[i], "-k")){
+        kb = TRUE;
+      } else if(!strcmp(argv[i], "-v")){
+        vb = TRUE;
+      } else if(!strcmp(argv[i], "-ox")){
         ox = TRUE;
       } else {
         usage();
@@ -407,7 +413,7 @@ int runlist(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = dolist(name, ox);
+  rv = dolist(name, kb, vb, ox);
   return rv;
 }
 
@@ -670,7 +676,7 @@ int doget(const char *name, const char *kbuf, int ksiz, int start, int max, int
 
 
 /* perform list command */
-int dolist(const char *name, int ox){
+int dolist(const char *name, int kb, int vb, int ox){
   DEPOT *depot;
   char *kbuf, *vbuf;
   int ksiz, vsiz;
@@ -686,13 +692,13 @@ int dolist(const char *name, int ox){
       break;
     }
     if(ox){
-      printobjhex(kbuf, ksiz);
-      putchar('\t');
-      printobjhex(vbuf, vsiz);
+      if(!vb) printobjhex(kbuf, ksiz);
+      if(!kb && !vb) putchar('\t');
+      if(!kb) printobjhex(vbuf, vsiz);
     } else {
-      printobj(kbuf, ksiz);
-      putchar('\t');
-      printobj(vbuf, vsiz);
+      if(!vb) printobj(kbuf, ksiz);
+      if(!kb && !vb) putchar('\t');
+      if(!kb) printobj(vbuf, vsiz);
     }
     putchar('\n');
     free(vbuf);
