@@ -104,7 +104,9 @@ public class Villa implements ADBM {
   /** write mode: keep the existing value */
   public static final int DKEEP = 1;
   /** write mode: concatenate values */
-  public static final int DDUP = 2;
+  public static final int DCAT = 2;
+  /** write mode: allow duplication of records */
+  public static final int DDUP = 3;
   //----------------------------------------------------------------
   // jump modes
   //----------------------------------------------------------------
@@ -236,7 +238,8 @@ public class Villa implements ADBM {
    * @param val a byte array of a value.
    * @param dmode behavior when the key overlaps, by the following values: `Villa.DOVER',
    * which means the specified value overwrites the existing one, `Villa.DKEEP', which means the
-   * existing value is kept, `Villa.DDUP', which means duplication of keys is allowed.
+   * existing value is kept, `Villa.DCAT', which means the specified value is concatenated at the
+   * end of the existing value, `Villa.DDUP', which means duplication of keys is allowed.
    * @throws VillaException if an error occurs or replace is cancelled.
    * @note A duplicated record is stored at the tail of the records of the same key.  The cursor
    * becomes unavailable due to updating database.
@@ -321,6 +324,32 @@ public class Villa implements ADBM {
     if(val == null) throw new VillaException();
     return val;
   }
+  /**
+   * Get the size of the value of a record.
+   * @param key a byte array of a key.
+   * @return the size of the value of the corresponding record.  If no record corresponds, -1 is
+   * returned.  If multiple records correspond, the size of the first is returned.
+   * @throws VillaException if an error occurs.
+   */
+  public int vsiz(byte[] key) throws VillaException {
+    if(index < 0) throw new VillaException();
+    synchronized(ADBM.class){
+      int rv = vlvsiz(index, key, key.length);
+      if(rv == -1) throw new VillaException(vlecode());
+      return rv;
+    }
+  }
+  /**
+   * Get the size of the value of a record, composed of serializable objects.
+   * The same as `vsiz(qdbm.Util.serialize(key))'.
+   * @see #vnum(byte[])
+   * @note If serialization is failed, an instance of `VillaException' is thrown.
+   */
+  public int vsizobj(Object key) throws VillaException {
+    byte[] kbuf = Util.serialize(key);
+    if(kbuf == null) throw new VillaException();
+    return vsiz(kbuf);
+  }
   /**
    * Get the number of records corresponding a key.
    * @param key a byte array of a key.
@@ -761,6 +790,7 @@ public class Villa implements ADBM {
                                                      byte[] val, int vsiz, int dmode);
   private static synchronized final native int vlout(int index, byte[] key, int ksiz);
   private static synchronized final native byte[] vlget(int index, byte[] key, int ksiz);
+  private static synchronized final native int vlvsiz(int index, byte[] key, int ksiz);
   private static synchronized final native int vlvnum(int index, byte[] key, int ksiz);
   private static synchronized final native int vlcurfirst(int index);
   private static synchronized final native int vlcurlast(int index);
