@@ -202,7 +202,7 @@ class VillaTester extends Thread {
       for(int i = 1; i <= LOOPNUM; i++){
         byte[] buf = Util.numstr(i, 8, '0').getBytes();
         byte[] val = villa.get(buf);
-        if(val.length != 1){
+        if(val.length != 1 || villa.vsiz(buf) != 1){
           err = true;
           System.err.println("size error: " + val.length);
         }
@@ -251,7 +251,8 @@ class VillaTester extends Thread {
           tran = true;
         }
         if(RND.nextInt(LOOPNUM / 20) == 0) yield();
-        myvilla.put(buf, (myid % 2 == 0 ? "=" : "*").getBytes(), Villa.DDUP);
+        myvilla.put(buf, (myid % 2 == 0 ? "=" : "*").getBytes(),
+                    RND.nextInt(3) == 0 ? Villa.DCAT : Villa.DDUP);
       } catch(VillaException e){
         try {
           if(tran) myvilla.tranabort();
