@@ -175,18 +175,17 @@ Java_qdbm_Villa_vlget(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
 {
   jbyte *kbuf;
   jboolean ick;
-  char *val;
+  const char *val;
   int vsiz;
   jbyteArray vbuf;
   vljnienv = env;
   vlmyclass = myclass;
   kbuf = (*env)->GetByteArrayElements(env, key, &ick);
-  val = vlget(vltable[index], kbuf, ksiz, &vsiz);
+  val = vlgetcache(vltable[index], kbuf, ksiz, &vsiz);
   if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
     (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
-    free(val);
   } else {
     vbuf = NULL;
   }
@@ -194,6 +193,21 @@ Java_qdbm_Villa_vlget(JNIEnv *env, jclass myclass, jint index, jbyteArray key, j
 }
 
 
+JNIEXPORT jint JNICALL
+Java_qdbm_Villa_vlvsiz(JNIEnv *env, jclass myclass, jint index, jbyteArray key, jint ksiz)
+{
+  jbyte *kbuf;
+  jboolean ick;
+  int rv;
+  vljnienv = env;
+  vlmyclass = myclass;
+  kbuf = (*env)->GetByteArrayElements(env, key, &ick);
+  rv = vlvsiz(vltable[index], kbuf, ksiz);
+  if(ick == JNI_TRUE) (*env)->ReleaseByteArrayElements(env, key, kbuf, JNI_ABORT);
+  return rv;
+}
+
+
 JNIEXPORT jint JNICALL
 Java_qdbm_Villa_vlvnum(JNIEnv *env, jclass myclass, jint index, jbyteArray key, jint ksiz)
 {
@@ -264,36 +278,34 @@ Java_qdbm_Villa_vlcurjump(JNIEnv *env, jclass myclass, jint index,
 JNIEXPORT jbyteArray JNICALL
 Java_qdbm_Villa_vlcurkey(JNIEnv *env, jclass myclass, jint index)
 {
-  char *val;
-  int vsiz;
-  jbyteArray vbuf;
+  const char *key;
+  int ksiz;
+  jbyteArray kbuf;
   vljnienv = env;
   vlmyclass = myclass;
-  val = vlcurkey(vltable[index], &vsiz);
-  if(val){
-    vbuf = (*env)->NewByteArray(env, vsiz);
-    (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
-    free(val);
+  key = vlcurkeycache(vltable[index], &ksiz);
+  if(key){
+    kbuf = (*env)->NewByteArray(env, ksiz);
+    (*env)->SetByteArrayRegion(env, kbuf, 0, ksiz, key);
   } else {
-    vbuf = NULL;
+    kbuf = NULL;
   }
-  return vbuf;
+  return kbuf;
 }
 
 
 JNIEXPORT jbyteArray JNICALL
 Java_qdbm_Villa_vlcurval(JNIEnv *env, jclass myclass, jint index)
 {
-  char *val;
+  const char *val;
   int vsiz;
   jbyteArray vbuf;
   vljnienv = env;
   vlmyclass = myclass;
-  val = vlcurval(vltable[index], &vsiz);
+  val = vlcurvalcache(vltable[index], &vsiz);
   if(val){
     vbuf = (*env)->NewByteArray(env, vsiz);
     (*env)->SetByteArrayRegion(env, vbuf, 0, vsiz, val);
-    free(val);
   } else {
     vbuf = NULL;
   }
