@@ -18,6 +18,36 @@
 
 
 
+/*************************************************************************************************
+ * for dosish filesystems
+ *************************************************************************************************/
+
+
+#if defined(_SYS_MSVC_) || defined(_SYS_MINGW_) || defined(_SYS_CYGWIN_)
+
+
+#define DOSPATHBUFSIZ  8192
+
+
+int _qdbm_win32_lstat(const char *pathname, struct stat *buf){
+  char pbuf[DOSPATHBUFSIZ], *p;
+  int inode;
+  if(stat(pathname, buf) == -1) return -1;
+  if(GetFullPathName(pathname, DOSPATHBUFSIZ, pbuf, &p) != 0){
+    inode = 11003;
+    for(p = pbuf; *p != '\0'; p++){
+      inode = inode * 31 + *(unsigned char *)p;
+    }
+    buf->st_ino = (inode * 911) & 0x7FFF;
+  }
+  return 0;
+}
+
+
+#endif
+
+
+
 /*************************************************************************************************
  * for POSIX thread
  *************************************************************************************************/
