@@ -157,22 +157,22 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   CBLIST *tlist;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if((tmp = vlget(odeum->rdocsdb, doc->uri, -1, &tsiz)) != NULL){
     if(!over){
       free(tmp);
-      dpecode = DP_EKEEP;
+      dpecodeset(DP_EKEEP, __FILE__, __LINE__);
       return FALSE;
     }
     if(tsiz != sizeof(int) || !odoutbyid(odeum, *(int *)tmp)){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return FALSE;
     }
@@ -212,7 +212,7 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   if(_qdbm_deflate){
     if(!(zbuf = _qdbm_deflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
-      dpecode = DP_EMISC;
+      dpecodeset(DP_EMISC, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return FALSE;
     }
@@ -222,7 +222,7 @@ int odput(ODEUM *odeum, ODDOC *doc, int wmax, int over){
   }
   if(!crput(odeum->docsdb, (char *)&docid, sizeof(int), tmp, tsiz, CR_DKEEP)){
     free(tmp);
-    if(dpecode == DP_EKEEP) dpecode = DP_EBROKEN;
+    if(dpecode == DP_EKEEP) dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return FALSE;
   }
@@ -278,11 +278,11 @@ int odout(ODEUM *odeum, const char *uri){
   int tsiz, docid;
   assert(odeum && uri);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(!(tmp = vlget(odeum->rdocsdb, uri, -1, &tsiz))){
@@ -291,7 +291,7 @@ int odout(ODEUM *odeum, const char *uri){
   }
   if(tsiz != sizeof(int)){
     free(tmp);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return FALSE;
   }
@@ -309,11 +309,11 @@ int odoutbyid(ODEUM *odeum, int id){
   CBMAP *map;
   assert(odeum && id > 0);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(!(tmp = crget(odeum->docsdb, (char *)&id, sizeof(int), 0, -1, &tsiz))){
@@ -323,7 +323,7 @@ int odoutbyid(ODEUM *odeum, int id){
   if(_qdbm_inflate){
     if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return FALSE;
     }
@@ -336,7 +336,7 @@ int odoutbyid(ODEUM *odeum, int id){
   uritmp = cbmapget(map, OD_URIEXPR, sizeof(OD_URIEXPR), &uritsiz);
   if(!uritmp || !vlout(odeum->rdocsdb, uritmp, uritsiz)){
     cbmapclose(map);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return FALSE;
   }
@@ -356,7 +356,7 @@ ODDOC *odget(ODEUM *odeum, const char *uri){
   int tsiz, docid;
   assert(odeum && uri);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   if(!(tmp = vlget(odeum->rdocsdb, uri, -1, &tsiz))){
@@ -365,7 +365,7 @@ ODDOC *odget(ODEUM *odeum, const char *uri){
   }
   if(tsiz != sizeof(int)){
     free(tmp);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return NULL;
   }
@@ -384,11 +384,11 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
   CBMAP *map;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   if(id < 1){
-    dpecode = DP_ENOITEM;
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
     return NULL;
   }
   if(!(tmp = crget(odeum->docsdb, (char *)&id, sizeof(int), 0, -1, &tsiz))){
@@ -398,7 +398,7 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
   if(_qdbm_inflate){
     if(!(zbuf = _qdbm_inflate(tmp, tsiz, &zsiz, _QDBM_ZMRAW))){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return NULL;
     }
@@ -414,7 +414,7 @@ ODDOC *odgetbyid(ODEUM *odeum, int id){
   awordstmp = cbmapget(map, OD_AWORDSEXPR, sizeof(OD_AWORDSEXPR), &awordstsiz);
   if(!uritmp || !attrstmp || !nwordstmp || !awordstmp){
     cbmapclose(map);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return NULL;
   }
@@ -442,7 +442,7 @@ int odgetidbyuri(ODEUM *odeum, const char *uri){
   int tsiz, docid;
   assert(odeum && uri);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   if(!(tmp = vlget(odeum->rdocsdb, uri, -1, &tsiz))){
@@ -451,7 +451,7 @@ int odgetidbyuri(ODEUM *odeum, const char *uri){
   }
   if(tsiz != sizeof(int)){
     free(tmp);
-    dpecode = DP_EBROKEN;
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     odeum->fatal = TRUE;
     return -1;
   }
@@ -465,11 +465,11 @@ int odgetidbyuri(ODEUM *odeum, const char *uri){
 int odcheck(ODEUM *odeum, int id){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(id < 1){
-    dpecode = DP_ENOITEM;
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
     return FALSE;
   }
   return crvsiz(odeum->docsdb, (char *)&id, sizeof(int)) != -1;
@@ -482,7 +482,7 @@ ODPAIR *odsearch(ODEUM *odeum, const char *word, int max, int *np){
   int tsiz;
   assert(odeum && word && np);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   if(odeum->wmode && cbmaprnum(odeum->sortmap) > 0 &&
@@ -509,7 +509,7 @@ int odsearchdnum(ODEUM *odeum, const char *word){
   int rv;
   assert(odeum && word);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   rv = crvsiz(odeum->indexdb, word, -1);
@@ -521,7 +521,7 @@ int odsearchdnum(ODEUM *odeum, const char *word){
 int oditerinit(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   return criterinit(odeum->docsdb);
@@ -535,7 +535,7 @@ ODDOC *oditernext(ODEUM *odeum){
   ODDOC *doc;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   doc = NULL;
@@ -546,7 +546,7 @@ ODDOC *oditernext(ODEUM *odeum){
     }
     if(tsiz != sizeof(int)){
       free(tmp);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       odeum->fatal = TRUE;
       return NULL;
     }
@@ -567,11 +567,11 @@ int odsync(ODEUM *odeum){
   char numbuf[OD_NUMBUFSIZ];
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(odotcb) odotcb("odsync", odeum, "writing meta information");
@@ -616,11 +616,11 @@ int odsync(ODEUM *odeum){
 int odoptimize(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odeum->wmode){
-    dpecode = DP_EMODE;
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
   if(!odcacheflush(odeum, "odoptimize")){
@@ -662,7 +662,7 @@ int odoptimize(ODEUM *odeum){
 char *odname(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return NULL;
   }
   return cbmemdup(odeum->name, -1);
@@ -674,7 +674,7 @@ double odfsiz(ODEUM *odeum){
   double fsiz, rv;
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   fsiz = 0;
@@ -692,7 +692,7 @@ double odfsiz(ODEUM *odeum){
 int odbnum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return crbnum(odeum->indexdb);
@@ -703,7 +703,7 @@ int odbnum(ODEUM *odeum){
 int odbusenum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return crbusenum(odeum->indexdb);
@@ -714,7 +714,7 @@ int odbusenum(ODEUM *odeum){
 int oddnum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return odeum->dnum;
@@ -725,7 +725,7 @@ int oddnum(ODEUM *odeum){
 int odwnum(ODEUM *odeum){
   assert(odeum);
   if(odeum->fatal){
-    dpecode = DP_EFATAL;
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
     return -1;
   }
   return crrnum(odeum->indexdb);
@@ -967,19 +967,19 @@ int odremove(const char *name){
   sprintf(docsname, "%s%c%s", name, MYPATHCHR, OD_DOCSNAME);
   sprintf(indexname, "%s%c%s", name, MYPATHCHR, OD_INDEXNAME);
   sprintf(rdocsname, "%s%c%s", name, MYPATHCHR, OD_RDOCSNAME);
-  if(stat(name, &sbuf) == -1){
-    dpecode = DP_ESTAT;
+  if(lstat(name, &sbuf) == -1){
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return FALSE;
   }
-  if(stat(docsname, &sbuf) != -1 && !crremove(docsname)) return FALSE;
-  if(stat(indexname, &sbuf) != -1 && !crremove(indexname)) return FALSE;
-  if(stat(rdocsname, &sbuf) != -1 && !vlremove(rdocsname)) return FALSE;
+  if(lstat(docsname, &sbuf) != -1 && !crremove(docsname)) return FALSE;
+  if(lstat(indexname, &sbuf) != -1 && !crremove(indexname)) return FALSE;
+  if(lstat(rdocsname, &sbuf) != -1 && !vlremove(rdocsname)) return FALSE;
   if((list = cbdirlist(name)) != NULL){
     for(i = 0; i < cblistnum(list); i++){
       file = cblistval(list, i, NULL);
       if(!strcmp(file, MYCDIRSTR) || !strcmp(file, MYPDIRSTR)) continue;
       sprintf(path, "%s%c%s", name, MYPATHCHR, file);
-      if(stat(path, &sbuf) == -1) continue;
+      if(lstat(path, &sbuf) == -1) continue;
       if(S_ISDIR(sbuf.st_mode)){
         if(!crremove(path)) return FALSE;
       } else {
@@ -989,7 +989,7 @@ int odremove(const char *name){
     cblistclose(list);
   }
   if(rmdir(name) == -1){
-    dpecode = DP_ERMDIR;
+    dpecodeset(DP_ERMDIR, __FILE__, __LINE__);
     return FALSE;
   }
   return TRUE;
@@ -1540,7 +1540,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
   ODEUM *odeum;
   assert(name);
   if(strlen(name) > OD_NAMEMAX){
-    dpecode = DP_EMISC;
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
     return NULL;
   }
   cromode = CR_OREADER;
@@ -1573,12 +1573,12 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
   rdocsdb = NULL;
   if((omode & OD_OWRITER) && (omode & OD_OCREAT)){
     if(mkdir(name, OD_DIRMODE) == -1 && errno != EEXIST){
-      dpecode = DP_EMKDIR;
+      dpecodeset(DP_EMKDIR, __FILE__, __LINE__);
       return NULL;
     }
   }
-  if(stat(name, &sbuf) == -1){
-    dpecode = DP_ESTAT;
+  if(lstat(name, &sbuf) == -1){
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
     return NULL;
   }
   inode = sbuf.st_ino;
@@ -1624,7 +1624,7 @@ static ODEUM *odopendb(const char *name, int omode, int docsbnum, int indexbnum,
       vlclose(rdocsdb);
       crclose(indexdb);
       crclose(docsdb);
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       return NULL;
     }
   } else {
@@ -1828,7 +1828,7 @@ static int odpurgeindex(ODEUM *odeum, const char *fname){
       break;
     }
     if(!(vbuf = crget(odeum->indexdb, kbuf, ksiz, 0, -1, &vsiz))){
-      dpecode = DP_EBROKEN;
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
       free(kbuf);
       return FALSE;
     }
