@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Utility for indexing document files into a database of Odeum
- *                                                      Copyright (C) 2000-2004 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -85,6 +85,7 @@ int runrelate(int argc, char **argv);
 int runpurge(int argc, char **argv);
 int bwimatchlist(const char *str, const CBLIST *keys);
 char *fgetl(FILE *ifp);
+void otcb(const char *fname, ODEUM *odeum, const char *msg);
 void pdperror(const char *name);
 void printferror(const char *format, ...);
 void printfinfo(const char *format, ...);
@@ -113,6 +114,7 @@ int main(int argc, char **argv){
   sigterm = FALSE;
   setsignals();
   if(argc < 2) usage();
+  odsetotcb(otcb);
   rv = 0;
   if(!strcmp(argv[1], "register")){
     rv = runregister(argc, argv);
@@ -282,6 +284,15 @@ char *fgetl(FILE *ifp){
 }
 
 
+/* report the outturn */
+void otcb(const char *fname, ODEUM *odeum, const char *msg){
+  char *name;
+  name = odname(odeum);
+  printf("%s: %s: %s: %s\n", progname, fname, name, msg);
+  free(name);
+}
+
+
 /* print an error message */
 void pdperror(const char *name){
   printf("%s: ERROR: %s: %s\n", progname, name, dperrmsg(dpecode));
