@@ -69,7 +69,7 @@ int doput(const char *name, const char *text, const char *uri, const char *title
           const char *author, const char *date, int wmax, int keep);
 int doout(const char *name, const char *uri, int id);
 int doget(const char *name, const char *uri, int id, int tb, int hb);
-int dosearch(const char *name, const char *text, int max, int or, int idf,
+int dosearch(const char *name, const char *text, int max, int or, int idf, int ql,
              int tb, int hb, int nb);
 int dolist(const char *name, int tb, int hb);
 int dooptimize(const char *name);
@@ -87,13 +87,15 @@ int main(int argc, char **argv){
   progname = argv[0];
   if((env = getenv("QDBMDBGFD")) != NULL) dpdbgfd = atoi(env);
   if(argc < 2) usage();
-  odsetotcb(otcb);
   rv = 0;
   if(!strcmp(argv[1], "create")){
+    odsetotcb(otcb);
     rv = runcreate(argc, argv);
   } else if(!strcmp(argv[1], "put")){
+    odsetotcb(otcb);
     rv = runput(argc, argv);
   } else if(!strcmp(argv[1], "out")){
+    odsetotcb(otcb);
     rv = runout(argc, argv);
   } else if(!strcmp(argv[1], "get")){
     rv = runget(argc, argv);
@@ -102,10 +104,12 @@ int main(int argc, char **argv){
   } else if(!strcmp(argv[1], "list")){
     rv = runlist(argc, argv);
   } else if(!strcmp(argv[1], "optimize")){
+    odsetotcb(otcb);
     rv = runoptimize(argc, argv);
   } else if(!strcmp(argv[1], "inform")){
     rv = runinform(argc, argv);
   } else if(!strcmp(argv[1], "merge")){
+    odsetotcb(otcb);
     rv = runmerge(argc, argv);
   } else if(!strcmp(argv[1], "remove")){
     rv = runremove(argc, argv);
@@ -327,13 +331,14 @@ int runget(int argc, char **argv){
 /* parse arguments of search command */
 int runsearch(int argc, char **argv){
   char *name, *srchwords[MAXSRCHWORDS];
-  int i, wnum, max, or, idf, tb, hb, nb, rv;
+  int i, wnum, max, or, idf, ql, tb, hb, nb, rv;
   CBDATUM *text;
   name = NULL;
   wnum = 0;
   max = -1;
   or = FALSE;
   idf = FALSE;
+  ql = FALSE;
   tb = FALSE;
   hb = FALSE;
   nb = FALSE;
@@ -346,6 +351,8 @@ int runsearch(int argc, char **argv){
         or = TRUE;
       } else if(!strcmp(argv[i], "-idf")){
         idf = TRUE;
+      } else if(!strcmp(argv[i], "-ql")){
+        ql = TRUE;
       } else if(!strcmp(argv[i], "-t")){
         tb = TRUE;
       } else if(!strcmp(argv[i], "-h")){
@@ -367,7 +374,7 @@ int runsearch(int argc, char **argv){
     if(i > 0) cbdatumcat(text, " ", 1);
     cbdatumcat(text, srchwords[i], -1);
   }
-  rv = dosearch(name, cbdatumptr(text), max, or, idf, tb, hb, nb);
+  rv = dosearch(name, cbdatumptr(text), max, or, idf, ql, tb, hb, nb);
   cbdatumclose(text);
   return rv;
 }
@@ -795,7 +802,7 @@ int doget(const char *name, const char *uri, int id, int tb, int hb){
 
 
 /* perform search command */
-int dosearch(const char *name, const char *text, int max, int or, int idf,
+int dosearch(const char *name, const char *text, int max, int or, int idf, int ql,
              int tb, int hb, int nb){
   ODEUM *odeum;
   CBLIST *awords, *nwords, *uris, *hits;
@@ -815,46 +822,50 @@ int dosearch(const char *name, const char *text, int max, int or, int idf,
   hits = cblistopen();
   last = NULL;
   lnum = 0;
-  for(i = 0; i < cblistnum(awords); i++){
-    asis = cblistval(awords, i, NULL);
-    normal = odnormalizeword(asis);
-    cblistpush(nwords, normal, -1);
-    if(strlen(normal) < 1){
-      free(normal);
-      continue;
-    }
-    if(!(pairs = odsearch(odeum, normal, or ? max : -1, &pnum))){
-      pdperror(name);
-      free(normal);
-      continue;
-    }
-    if((hnum = odsearchdnum(odeum, normal)) < 0) hnum = 0;
-    if(idf){
-      ival = odlogarithm(hnum);
-      ival = (ival * ival) / 4.0;
-      if(ival < 4.0) ival = 4.0;
-      for(j = 0; j < pnum; j++){
-        pairs[j].score = (int)(pairs[j].score / ival);
+  if(ql){
+    last= odquery(odeum, text, &lnum, NULL);
+  } else {
+    for(i = 0; i < cblistnum(awords); i++){
+      asis = cblistval(awords, i, NULL);
+      normal = odnormalizeword(asis);
+      cblistpush(nwords, normal, -1);
+      if(strlen(normal) < 1){
+        free(normal);
+        continue;
       }
-    }
-    cblistpush(uris, normal, -1);
-    sprintf(numbuf, "%d", hnum);
-    cblistpush(hits, numbuf, -1);
-    if(last){
-      if(or){
-        tmp = odpairsor(last, lnum, pairs, pnum, &tnum);
+      if(!(pairs = odsearch(odeum, normal, or ? max : -1, &pnum))){
+        pdperror(name);
+        free(normal);
+        continue;
+      }
+      if((hnum = odsearchdnum(odeum, normal)) < 0) hnum = 0;
+      if(idf){
+        ival = odlogarithm(hnum);
+        ival = (ival * ival) / 4.0;
+        if(ival < 4.0) ival = 4.0;
+        for(j = 0; j < pnum; j++){
+          pairs[j].score = (int)(pairs[j].score / ival);
+        }
+      }
+      cblistpush(uris, normal, -1);
+      sprintf(numbuf, "%d", hnum);
+      cblistpush(hits, numbuf, -1);
+      if(last){
+        if(or){
+          tmp = odpairsor(last, lnum, pairs, pnum, &tnum);
+        } else {
+          tmp = odpairsand(last, lnum, pairs, pnum, &tnum);
+        }
+        free(last);
+        free(pairs);
+        last = tmp;
+        lnum = tnum;
       } else {
-        tmp = odpairsand(last, lnum, pairs, pnum, &tnum);
+        last = pairs;
+        lnum = pnum;
       }
-      free(last);
-      free(pairs);
-      last = tmp;
-      lnum = tnum;
-    } else {
-      last = pairs;
-      lnum = pnum;
+      free(normal);
     }
-    free(normal);
   }
   if(hb){
     printf("TOTAL: %d\n", lnum);
