@@ -42,7 +42,8 @@ use constant CMPDEC => 1;                # compare decimal strings
 
 use constant DOVER => 0;                 # overwrite an existing value
 use constant DKEEP => 1;                 # keep an existing value
-use constant DDUP => 2;                  # allow duplication of records
+use constant DCAT => 2;                  # concatenate values
+use constant DDUP => 3;                  # allow duplication of records
 
 use constant JFORWARD => 0;              # step forward
 use constant JBACKWARD => 1;             # step backward
@@ -139,7 +140,8 @@ sub close {
 # `$val' specifies a value.  If it is undef, this method has no effect.
 # `$dmode' specifies behavior when the key overlaps, by the following values: `Villa::DOVER',
 # which means the specified value overwrites the existing one, `Villa::DKEEP', which means the
-# existing value is kept, `Villa::DDUP', which means duplication of keys is allowed.  If it is
+# existing value is kept, `Villa::DCAT', which means the specified value is concatenated at the
+# end of the existing value, `Villa::DDUP', which means duplication of keys is allowed.  If it is
 # undef, `Villa::DOVER' is specified.
 # If successful, the return value is true, else, it is false.
 # A duplicated record is stored at the tail of the records of the same key.  The cursor becomes
@@ -230,6 +232,32 @@ sub get {
 }
 
 
+##
+# $num = $villa->vsiz($key);
+# Method: Get the size of the value of a record.
+# `$key' specifies a key.  If it is undef, this method has no effect.
+# The return value is the size of the value of the corresponding record.  If no record
+# corresponds, -1 is returned.  If multiple records correspond, the size of the first is
+# returned.
+#
+sub vsiz {
+    my($self) = shift;
+    ($$self[1]) || return 0;
+    my($key) = shift;
+    (scalar(@_) == 0) || return 0;
+    (defined($key)) || return 0;
+    my($villa) = $handles{$$self[0]};
+    if($$self[2]){
+        local($_) = $key;
+        $$self[2]();
+        $key = $_;
+    }
+    my($rv) = plvlvsiz($villa, $key, length($key));
+    $errmsg = plvlerrmsg();
+    return $rv;
+}
+
+
 ##
 # $num = $villa->vnum($key);
 # Method: Get the number of records corresponding a key.
