@@ -143,6 +143,7 @@ const int Villa::OLCKNB = VL_OLCKNB;
 const int Villa::OZCOMP = VL_OZCOMP;
 const int Villa::DOVER = VL_DOVER;
 const int Villa::DKEEP = VL_DKEEP;
+const int Villa::DCAT = VL_DCAT;
 const int Villa::DDUP = VL_DDUP;
 const int Villa::JFORWARD = VL_JFORWARD;
 const int Villa::JBACKWARD = VL_JBACKWARD;
@@ -254,6 +255,16 @@ char* Villa::get(const char* kbuf, int ksiz, int* sp) throw(Villa_error){
 }
 
 
+int Villa::vsiz(const char *kbuf, int ksiz) throw(Villa_error){
+  int vnum;
+  if(!villa) throw Villa_error();
+  if(pthread_mutex_lock(&mutex) != 0) throw Villa_error();
+  vnum = vlvsiz(villa, kbuf, ksiz);
+  pthread_mutex_unlock(&mutex);
+  return vnum;
+}
+
+
 int Villa::vnum(const char *kbuf, int ksiz) throw(Villa_error){
   int vnum;
   if(!villa) throw Villa_error();
