@@ -155,7 +155,8 @@ public:
   static const int OZCOMP;            ///< open mode: compress leaves
   static const int DOVER;             ///< write mode: overwrite the existing value
   static const int DKEEP;             ///< write mode: keep the existing value
-  static const int DDUP;              ///< write mode: concatenate values
+  static const int DCAT;              ///< write mode: concatenate values
+  static const int DDUP;              ///< write mode: allow duplication of records
   static const int JFORWARD;          ///< jump mode: step forward
   static const int JBACKWARD;         ///< jump mode: step backward
   //----------------------------------------------------------------
@@ -264,7 +265,8 @@ public:
    * with `std::strlen(vbuf)'.
    * @param dmode behavior when the key overlaps, by the following values: `Villa::DOVER',
    * which means the specified value overwrites the existing one, `Villa::DKEEP', which means
-   * the existing value is kept, `Villa::DDUP', which means duplication of keys is allowed.
+   * the existing value is kept, `Villa::DCAT', which means the specified value is concatenated
+   * at the end of the existing value, `Villa::DDUP', which means duplication of keys is allowed.
    * @throw Villa_error if an error occurs or replace is cancelled.
    * @note A duplicated record is stored at the tail of the records of the same key.  The
    * cursor becomes unavailable due to updating database.
@@ -297,6 +299,16 @@ public:
    * should be released with the `std::free' call if it is no longer in use.
    */
   virtual char* get(const char* kbuf, int ksiz, int* sp = 0) throw(Villa_error);
+  /**
+   * Get the size of the value of a record.
+   * @param kbuf the pointer to the region of a key.
+   * @param ksiz the size of the region of the key.  If it is negative, the size is assigned
+   * with `std::strlen(kbuf)'.
+   * @return the size of the value of the corresponding record.  If no record corresponds, -1 is
+   * returned.  If multiple records correspond, the size of the first is returned.
+   * @throw Villa_error if an error occurs.
+   */
+  virtual int vsiz(const char* kbuf, int ksiz) throw(Villa_error);
   /**
    * Get the number of records corresponding a key.
    * @param kbuf the pointer to the region of a key.
