@@ -142,8 +142,9 @@ class Villa
   # `val' specifies a value.  Although it must be an instance of String, binary data is okey.
   # `dmode' specifies behavior when the key overlaps, by the following values: `Villa::DOVER',
   # which means the specified value overwrites the existing one, `Villa::DKEEP', which means
-  # the existing value is kept, `Villa::DDUP', which means duplication of keys is allowed.  If
-  # it is omitted `Villa::DOVER' is specified.
+  # the existing value is kept, `Villa::DCAT', which means the specified value is concatenated
+  # at the end of the existing value, `Villa::DDUP', which means duplication of keys is allowed.
+  # If it is omitted `Villa::DOVER' is specified.
   # The return value is always `true'.
   # An exception of `Villa::EANY' or its sub classes is thrown if an error occurs or replace
   # is cancelled.
@@ -237,6 +238,21 @@ class Villa
   #
   alias [] get
   ##
+  # num = villa.vsiz(key)
+  # Method: Get the size of the value of a record.
+  # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
+  # The return value is the size of the value of the corresponding record.  If no record
+  # corresponds, -1 is returned.  If multiple records correspond, the size of the first is
+  # returned.
+  # An exception of `Villa::EANY' or its sub classes is thrown if an error occurs.
+  #
+  def vsiz(key)
+    if(@cmode == CMPOBJ)
+      key = Marshal.dump(key)
+    end
+    mod_vsiz(@index, key)
+  end
+  ##
   # num = villa.vnum(key)
   # Method: Get the number of records corresponding a key.
   # `key' specifies a key.  Although it must be an instance of String, binary data is okey.
