@@ -65,6 +65,7 @@ static VALUE rbvlclose(VALUE vself, VALUE vindex);
 static VALUE rbvlput(VALUE vself, VALUE vindex, VALUE vkey, VALUE vval, VALUE vdmode);
 static VALUE rbvlout(VALUE vself, VALUE vindex, VALUE vkey);
 static VALUE rbvlget(VALUE vself, VALUE vindex, VALUE vkey);
+static VALUE rbvlvsiz(VALUE vself, VALUE vindex, VALUE vkey);
 static VALUE rbvlvnum(VALUE vself, VALUE vindex, VALUE vkey);
 static VALUE rbvlcurfirst(VALUE vself, VALUE vindex);
 static VALUE rbvlcurlast(VALUE vself, VALUE vindex);
@@ -151,6 +152,7 @@ Init_mod_villa(){
   rb_define_const(mvilla, "CMPOBJ", INT2FIX(2));
   rb_define_const(mvilla, "DOVER", INT2FIX(VL_DOVER));
   rb_define_const(mvilla, "DKEEP", INT2FIX(VL_DKEEP));
+  rb_define_const(mvilla, "DCAT", INT2FIX(VL_DCAT));
   rb_define_const(mvilla, "DDUP", INT2FIX(VL_DDUP));
   rb_define_const(mvilla, "JFORWARD", INT2FIX(VL_JFORWARD));
   rb_define_const(mvilla, "JBACKWARD", INT2FIX(VL_JBACKWARD));
@@ -159,6 +161,7 @@ Init_mod_villa(){
   rb_define_module_function(mvilla, "mod_put", rbvlput, 4);
   rb_define_module_function(mvilla, "mod_out", rbvlout, 2);
   rb_define_module_function(mvilla, "mod_get", rbvlget, 2);
+  rb_define_module_function(mvilla, "mod_vsiz", rbvlvsiz, 2);
   rb_define_module_function(mvilla, "mod_vnum", rbvlvnum, 2);
   rb_define_module_function(mvilla, "mod_curfirst", rbvlcurfirst, 1);
   rb_define_module_function(mvilla, "mod_curlast", rbvlcurlast, 1);
@@ -353,8 +356,7 @@ static VALUE rbvlout(VALUE vself, VALUE vindex, VALUE vkey){
 
 static VALUE rbvlget(VALUE vself, VALUE vindex, VALUE vkey){
   VILLA *villa;
-  const char *kbuf;
-  char *vbuf;
+  const char *kbuf, *vbuf;
   int index, ksiz, vsiz;
   VALUE vval;
   FIXNUM_P(vindex);
@@ -364,11 +366,24 @@ static VALUE rbvlget(VALUE vself, VALUE vindex, VALUE vkey){
   villa = vltable[index];
   if(!(vbuf = vlget(villa, kbuf, ksiz, &vsiz))) myerror(dpecode);
   vval = rb_str_new(vbuf, vsiz);
-  free(vbuf);
   return vval;
 }
 
 
+static VALUE rbvlvsiz(VALUE vself, VALUE vindex, VALUE vkey){
+  VILLA *villa;
+  const char *kbuf;
+  int index, ksiz, vsiz;
+  FIXNUM_P(vindex);
+  if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
+  kbuf = STR2CSTR(vkey);
+  ksiz = RSTRING(vkey)->len;
+  villa = vltable[index];
+  vsiz = vlvsiz(villa, kbuf, ksiz);
+  return INT2FIX(vsiz);
+}
+
+
 static VALUE rbvlvnum(VALUE vself, VALUE vindex, VALUE vkey){
   VILLA *villa;
   const char *kbuf;
@@ -447,30 +462,28 @@ static VALUE rbvlcurjump(VALUE vself, VALUE vindex, VALUE vkey, VALUE vjmode){
 
 static VALUE rbvlcurkey(VALUE vself, VALUE vindex){
   VILLA *villa;
-  char *kbuf;
+  const char *kbuf;
   int index, ksiz;
   VALUE vkey;
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!(kbuf = vlcurkey(villa, &ksiz))) myerror(dpecode);
+  if(!(kbuf = vlcurkeycache(villa, &ksiz))) myerror(dpecode);
   vkey = rb_str_new(kbuf, ksiz);
-  free(kbuf);
   return vkey;
 }
 
 
 static VALUE rbvlcurval(VALUE vself, VALUE vindex){
   VILLA *villa;
-  char *vbuf;
+  const char *vbuf;
   int index, vsiz;
   VALUE vval;
   FIXNUM_P(vindex);
   if((index = FIX2INT(vindex)) == -1) myerror(DP_EMISC);
   villa = vltable[index];
-  if(!(vbuf = vlcurval(villa, &vsiz))) myerror(dpecode);
+  if(!(vbuf = vlcurvalcache(villa, &vsiz))) myerror(dpecode);
   vval = rb_str_new(vbuf, vsiz);
-  free(vbuf);
   return vval;
 }
 
