@@ -337,6 +337,24 @@ char *vlget(VILLA *villa, const char *kbuf, int ksiz, int *sp){
 }
 
 
+/* Get the size of the value of a record. */
+int vlvsiz(VILLA *villa, const char *kbuf, int ksiz){
+  VLLEAF *leaf;
+  VLREC *recp;
+  int pid;
+  assert(villa && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((pid = vlsearchleaf(villa, kbuf, ksiz, NULL, NULL)) == -1) return -1;
+  if(!(leaf = vlleafload(villa, pid))) return -1;
+  if(!(recp = vlrecsearch(villa, leaf, kbuf, ksiz, NULL))){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return -1;
+  }
+  if(!villa->tran && !vlcacheadjust(villa)) return -1;
+  return CB_DATUMSIZE(recp->first);
+}
+
+
 /* Get the number of records corresponding a key. */
 int vlvnum(VILLA *villa, const char *kbuf, int ksiz){
   VLLEAF *leaf;
@@ -706,8 +724,8 @@ char *vlcurkey(VILLA *villa, int *sp){
 char *vlcurval(VILLA *villa, int *sp){
   VLLEAF *leaf;
   VLREC *recp;
-  const char *kbuf;
-  int ksiz;
+  const char *vbuf;
+  int vsiz;
   assert(villa);
   if(villa->curleaf == -1){
     dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
@@ -719,13 +737,13 @@ char *vlcurval(VILLA *villa, int *sp){
   }
   recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
   if(villa->curvnum < 1){
-    kbuf = CB_DATUMPTR(recp->first);
-    ksiz = CB_DATUMSIZE(recp->first);
+    vbuf = CB_DATUMPTR(recp->first);
+    vsiz = CB_DATUMSIZE(recp->first);
   } else {
-    kbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &ksiz);
+    vbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &vsiz);
   }
-  if(sp) *sp = ksiz;
-  return cbmemdup(kbuf, ksiz);
+  if(sp) *sp = vsiz;
+  return cbmemdup(vbuf, vsiz);
 }
 
 
@@ -1190,6 +1208,81 @@ int vlimportdb(VILLA *villa, const char *name){
 
 
 
+/*************************************************************************************************
+ * features for experts
+ *************************************************************************************************/
+
+
+/* Refer to a volatile cache of a value of a record. */
+const char *vlgetcache(VILLA *villa, const char *kbuf, int ksiz, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  int pid;
+  assert(villa && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((pid = vlsearchleaf(villa, kbuf, ksiz, NULL, NULL)) == -1) return NULL;
+  if(!(leaf = vlleafload(villa, pid))) return NULL;
+  if(!(recp = vlrecsearch(villa, leaf, kbuf, ksiz, NULL))){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return NULL;
+  }
+  if(!villa->tran && !vlcacheadjust(villa)) return NULL;
+  if(sp) *sp = CB_DATUMSIZE(recp->first);
+  return CB_DATUMPTR(recp->first);
+}
+
+
+/* Refer to volatile cache of the key of the record where the cursor is. */
+const char *vlcurkeycache(VILLA *villa, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  const char *kbuf;
+  int ksiz;
+  assert(villa);
+  if(villa->curleaf == -1){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!(leaf = vlleafload(villa, villa->curleaf))){
+    villa->curleaf = -1;
+    return FALSE;
+  }
+  recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
+  kbuf = CB_DATUMPTR(recp->key);
+  ksiz = CB_DATUMSIZE(recp->key);
+  if(sp) *sp = ksiz;
+  return kbuf;
+}
+
+
+/* Refer to volatile cache of the value of the record where the cursor is. */
+const char *vlcurvalcache(VILLA *villa, int *sp){
+  VLLEAF *leaf;
+  VLREC *recp;
+  const char *vbuf;
+  int vsiz;
+  assert(villa);
+  if(villa->curleaf == -1){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!(leaf = vlleafload(villa, villa->curleaf))){
+    villa->curleaf = -1;
+    return FALSE;
+  }
+  recp = (VLREC *)CB_LISTVAL(leaf->recs, villa->curknum, NULL);
+  if(villa->curvnum < 1){
+    vbuf = CB_DATUMPTR(recp->first);
+    vsiz = CB_DATUMSIZE(recp->first);
+  } else {
+    vbuf = CB_LISTVAL2(recp->rest, villa->curvnum - 1, &vsiz);
+  }
+  if(sp) *sp = vsiz;
+  return vbuf;
+}
+
+
+
 /*************************************************************************************************
  * private objects
  *************************************************************************************************/
@@ -1591,11 +1684,14 @@ static int vlleafaddrec(VILLA *villa, VLLEAF *leaf, int dmode,
     if(rv == 0){
       switch(dmode){
       case VL_DOVER:
-        cbdatumclose(recp->first);
-        recp->first = cbdatumopen(vbuf, vsiz);
+        cbdatumsetsize(recp->first, 0);
+        cbdatumcat(recp->first, vbuf, vsiz);
         break;
       case VL_DKEEP:
         return FALSE;
+      case VL_DCAT:
+        cbdatumcat(recp->first, vbuf, vsiz);
+        break;
       default:
         if(!recp->rest) recp->rest = cblistopen();
         cblistpush(recp->rest, vbuf, vsiz);
