@@ -63,7 +63,7 @@ int doput(const char *name, const char *kbuf, int ksiz, const char *vbuf, int vs
 int doout(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb);
 int doget(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb, int ox, int nb);
 int dolist(const char *name, const char *tbuf, int tsiz, const char *bbuf, int bsiz,
-             VLCFUNC cmp, int ki, int ox, int gt, int lt, int max, int desc);
+           VLCFUNC cmp, int ki, int kb, int vb, int ox, int gt, int lt, int max, int desc);
 int dooptimize(const char *name);
 int doinform(const char *name);
 int doremove(const char *name);
@@ -121,11 +121,11 @@ void usage(void){
   fprintf(stderr, "\n");
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s create [-zc] name\n", progname);
-  fprintf(stderr, "  %s put [-kx|-ki] [-vx|-vi|-vf] [-keep|-dup] name key val\n", progname);
+  fprintf(stderr, "  %s put [-kx|-ki] [-vx|-vi|-vf] [-keep|-cat|-dup] name key val\n", progname);
   fprintf(stderr, "  %s out [-l] [-kx|-ki] name key\n", progname);
   fprintf(stderr, "  %s get [-l] [-kx|-ki] [-ox] [-n] name key\n", progname);
-  fprintf(stderr, "  %s list [-kx|-ki] [-ox] [-top key] [-bot key] [-gt] [-lt] [-max num] "
-          "[-desc] name\n", progname);
+  fprintf(stderr, "  %s list [-k|-v] [-kx|-ki] [-ox] [-top key] [-bot key] [-gt] [-lt]"
+          " [-max num] [-desc] name\n", progname);
   fprintf(stderr, "  %s optimize name\n", progname);
   fprintf(stderr, "  %s inform name\n", progname);
   fprintf(stderr, "  %s remove name\n", progname);
@@ -225,6 +225,8 @@ int runput(int argc, char **argv){
         vf = TRUE;
       } else if(!strcmp(argv[i], "-keep")){
         dmode = VL_DKEEP;
+      } else if(!strcmp(argv[i], "-cat")){
+        dmode = VL_DCAT;
       } else if(!strcmp(argv[i], "-dup")){
         dmode = VL_DDUP;
       } else {
@@ -380,8 +382,10 @@ int runget(int argc, char **argv){
 /* parse arguments of list command */
 int runlist(int argc, char **argv){
   char *name, *top, *bot, *tbuf, *bbuf, *nstr;
-  int i, kx, ki, ox, gt, lt, max, desc, tsiz, bsiz, rv;
+  int i, kb, vb, kx, ki, ox, gt, lt, max, desc, tsiz, bsiz, rv;
   name = NULL;
+  kb = FALSE;
+  vb = FALSE;
   kx = FALSE;
   ki = FALSE;
   ox = FALSE;
@@ -394,7 +398,11 @@ int runlist(int argc, char **argv){
   nstr = NULL;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-kx")){
+      if(!strcmp(argv[i], "-k")){
+        kb = TRUE;
+      } else if(!strcmp(argv[i], "-v")){
+        vb = TRUE;
+      } else if(!strcmp(argv[i], "-kx")){
         kx = TRUE;
       } else if(!strcmp(argv[i], "-ki")){
         ki = TRUE;
@@ -443,7 +451,8 @@ int runlist(int argc, char **argv){
       bsiz = strlen(bbuf);
     }
   }
-  rv = dolist(name, tbuf, tsiz, bbuf, bsiz, ki ? VL_CMPINT : VL_CMPLEX, ki, ox, gt, lt, max, desc);
+  rv = dolist(name, tbuf, tsiz, bbuf, bsiz, ki ? VL_CMPINT : VL_CMPLEX, ki,
+              kb, vb, ox, gt, lt, max, desc);
   free(tbuf);
   free(bbuf);
   return rv;
@@ -732,7 +741,7 @@ int doget(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb, int
 
 /* perform list command */
 int dolist(const char *name, const char *tbuf, int tsiz, const char *bbuf, int bsiz,
-             VLCFUNC cmp, int ki, int ox, int gt, int lt, int max, int desc){
+           VLCFUNC cmp, int ki, int kb, int vb, int ox, int gt, int lt, int max, int desc){
   VILLA *villa;
   char *kbuf, *vbuf;
   int ksiz, vsiz, show, rv;
@@ -769,13 +778,13 @@ int dolist(const char *name, const char *tbuf, int tsiz, const char *bbuf, int b
         break;
       }
       if(ox){
-        printobjhex(kbuf, ksiz);
-        putchar('\t');
-        printobjhex(vbuf, vsiz);
+        if(!vb) printobjhex(kbuf, ksiz);
+        if(!kb && !vb) putchar('\t');
+        if(!kb) printobjhex(vbuf, vsiz);
       } else {
-        printobj(kbuf, ksiz);
-        putchar('\t');
-        printobj(vbuf, vsiz);
+        if(!vb) printobj(kbuf, ksiz);
+        if(!kb && !vb) putchar('\t');
+        if(!kb) printobj(vbuf, vsiz);
       }
       putchar('\n');
       free(kbuf);
@@ -811,13 +820,13 @@ int dolist(const char *name, const char *tbuf, int tsiz, const char *bbuf, int b
         break;
       }
       if(ox){
-        printobjhex(kbuf, ksiz);
-        putchar('\t');
-        printobjhex(vbuf, vsiz);
+        if(!vb) printobjhex(kbuf, ksiz);
+        if(!kb && !vb) putchar('\t');
+        if(!kb) printobjhex(vbuf, vsiz);
       } else {
-        printobj(kbuf, ksiz);
-        putchar('\t');
-        printobj(vbuf, vsiz);
+        if(!vb) printobj(kbuf, ksiz);
+        if(!kb && !vb) putchar('\t');
+        if(!kb) printobj(vbuf, vsiz);
       }
       putchar('\n');
       free(kbuf);
