@@ -55,11 +55,11 @@ void pdperror(const char *name);
 int myrand(void);
 int dowrite(const char *name, int rnum, int ii, int zc,
             int lrecmax, int nidxmax, int lcnum, int ncnum);
-int doread(const char *name, int ii);
-int dordup(const char *name, int rnum, int pnum, int ii, int zc,
+int doread(const char *name, int ii, int vc);
+int dordup(const char *name, int rnum, int pnum, int ii, int zc, int cc,
            int lrecmax, int nidxmax, int lcnum, int ncnum);
 int docombo(const char *name, int zc);
-int dowicked(const char *name, int rnum, int zc);
+int dowicked(const char *name, int rnum, int cb, int zc);
 
 
 /* main routine */
@@ -95,11 +95,11 @@ void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write [-int] [-zc] [-tune lrecmax nidxmax lcnum ncnum] name rnum\n",
           progname);
-  fprintf(stderr, "  %s read [-int] name\n", progname);
-  fprintf(stderr, "  %s rdup [-int] [-zc] [-tune lrecmax nidxmax lcnum ncnum] name rnum pnum\n",
-          progname);
+  fprintf(stderr, "  %s read [-int] [-vc] name\n", progname);
+  fprintf(stderr, "  %s rdup [-int] [-zc] [-cc] [-tune lrecmax nidxmax lcnum ncnum]"
+          " name rnum pnum\n", progname);
   fprintf(stderr, "  %s combo [-zc] name\n", progname);
-  fprintf(stderr, "  %s wicked [-zc] name rnum\n", progname);
+  fprintf(stderr, "  %s wicked [-c] [-zc] name rnum\n", progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -155,13 +155,16 @@ int runwrite(int argc, char **argv){
 /* parse arguments of read command */
 int runread(int argc, char **argv){
   char *name;
-  int i, ii, rv;
+  int i, ii, vc, rv;
   name = NULL;
   ii = FALSE;
+  vc = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-int")){
         ii = TRUE;
+      } else if(!strcmp(argv[i], "-vc")){
+        vc = TRUE;
       } else {
         usage();
       }
@@ -172,7 +175,7 @@ int runread(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = doread(name, ii);
+  rv = doread(name, ii, vc);
   return rv;
 }
 
@@ -180,7 +183,7 @@ int runread(int argc, char **argv){
 /* parse arguments of rdup command */
 int runrdup(int argc, char **argv){
   char *name, *rstr, *pstr;
-  int i, rnum, pnum, ii, zc, lrecmax, nidxmax, lcnum, ncnum, rv;
+  int i, rnum, pnum, ii, zc, cc, lrecmax, nidxmax, lcnum, ncnum, rv;
   name = NULL;
   rstr = NULL;
   pstr = NULL;
@@ -188,6 +191,7 @@ int runrdup(int argc, char **argv){
   pnum = 0;
   ii = FALSE;
   zc = FALSE;
+  cc = FALSE;
   lrecmax = -1;
   nidxmax = -1;
   lcnum = -1;
@@ -198,6 +202,8 @@ int runrdup(int argc, char **argv){
         ii = TRUE;
       } else if(!strcmp(argv[i], "-zc")){
         zc = TRUE;
+      } else if(!strcmp(argv[i], "-cc")){
+        cc = TRUE;
       } else if(!strcmp(argv[i], "-tune")){
         if(++i >= argc) usage();
         lrecmax = atoi(argv[i]);
@@ -224,7 +230,7 @@ int runrdup(int argc, char **argv){
   rnum = atoi(rstr);
   pnum = atoi(pstr);
   if(rnum < 1 || pnum < 1) usage();
-  rv = dordup(name, rnum, pnum, ii, zc, lrecmax, nidxmax, lcnum, ncnum);
+  rv = dordup(name, rnum, pnum, ii, zc, cc, lrecmax, nidxmax, lcnum, ncnum);
   return rv;
 }
 
@@ -257,13 +263,16 @@ int runcombo(int argc, char **argv){
 /* parse arguments of wicked command */
 int runwicked(int argc, char **argv){
   char *name, *rstr;
-  int i, zc, rnum, rv;
+  int i, cb, zc, rnum, rv;
   name = NULL;
   rstr = NULL;
+  cb = FALSE;
   zc = FALSE;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-zc")){
+      if(!strcmp(argv[i], "-c")){
+        cb = TRUE;
+      } else if(!strcmp(argv[i], "-zc")){
         zc = TRUE;
       } else {
         usage();
@@ -279,7 +288,7 @@ int runwicked(int argc, char **argv){
   if(!name || !rstr) usage();
   rnum = atoi(rstr);
   if(rnum < 1) usage();
-  rv = dowicked(name, rnum, zc);
+  rv = dowicked(name, rnum, cb, zc);
   return rv;
 }
 
@@ -366,9 +375,10 @@ int dowrite(const char *name, int rnum, int ii, int zc,
 
 
 /* perform read command */
-int doread(const char *name, int ii){
+int doread(const char *name, int ii, int vc){
   VILLA *villa;
   int i, rnum, err, len;
+  const char *cval;
   char buf[RECBUFSIZ], *val;
   printfflush("<Reading Test>\n  name=%s  int=%d\n\n", name, ii);
   /* open a database */
@@ -383,20 +393,36 @@ int doread(const char *name, int ii){
   for(i = 1; i <= rnum; i++){
     /* retrieve a record */
     if(ii){
-      if(!(val = vlget(villa, (char *)&i, sizeof(int), NULL))){
-        pdperror(name);
-        err = TRUE;
-        break;
+      if(vc){
+        if(!(cval = vlgetcache(villa, (char *)&i, sizeof(int), NULL))){
+          pdperror(name);
+          err = TRUE;
+          break;
+        }
+      } else {
+        if(!(val = vlget(villa, (char *)&i, sizeof(int), NULL))){
+          pdperror(name);
+          err = TRUE;
+          break;
+        }
+        free(val);
       }
-      free(val);
     } else {
       len = sprintf(buf, "%08d", i);
-      if(!(val = vlget(villa, buf, len, NULL))){
-        pdperror(name);
-        err = TRUE;
-        break;
+      if(vc){
+        if(!(cval = vlgetcache(villa, buf, len, NULL))){
+          pdperror(name);
+          err = TRUE;
+          break;
+        }
+      } else {
+        if(!(val = vlget(villa, buf, len, NULL))){
+          pdperror(name);
+          err = TRUE;
+          break;
+        }
+        free(val);
       }
-      free(val);
     }
     /* print progression */
     if(rnum > 250 && i % (rnum / 250) == 0){
@@ -418,10 +444,10 @@ int doread(const char *name, int ii){
 
 
 /* perform rdup command */
-int dordup(const char *name, int rnum, int pnum, int ii, int zc,
+int dordup(const char *name, int rnum, int pnum, int ii, int zc, int cc,
            int lrecmax, int nidxmax, int lcnum, int ncnum){
   VILLA *villa;
-  int i, omode, err, vi, len;
+  int i, omode, err, dmode, vi, len;
   char buf[RECBUFSIZ];
   printfflush("<Random Writing Test>\n  name=%s  rnum=%d  int=%d  zc=%d  "
               "lrecmax=%d  nidxmax=%d  lcnum=%d  ncnum=%d\n\n",
@@ -435,16 +461,18 @@ int dordup(const char *name, int rnum, int pnum, int ii, int zc,
   err = FALSE;
   if(lrecmax > 0) vlsettuning(villa, lrecmax, nidxmax, lcnum, ncnum);
   for(i = 1; i <= rnum; i++){
+    dmode = VL_DDUP;
+    if(cc && myrand() % 2 == 0) dmode = VL_DCAT;
     vi = myrand() % pnum + 1;
     if(ii){
-      if(!vlput(villa, (char *)&vi, sizeof(int), (char *)&vi, sizeof(int), VL_DDUP)){
+      if(!vlput(villa, (char *)&vi, sizeof(int), (char *)&vi, sizeof(int), dmode)){
         pdperror(name);
         err = TRUE;
         break;
       }
     } else {
       len = sprintf(buf, "%08d", vi);
-      if(!vlput(villa, buf, len, buf, len, VL_DDUP)){
+      if(!vlput(villa, buf, len, buf, len, dmode)){
         pdperror(name);
         err = TRUE;
         break;
@@ -506,7 +534,7 @@ int docombo(const char *name, int zc){
       return 1;
     }
     free(vbuf);
-    if(vsiz != 8){
+    if(vsiz != 8 || vlvsiz(villa, buf, len) != 8){
       fprintf(stderr, "%s: %s: invalid vsiz\n", progname, name);
       vlclose(villa);
       return 1;
@@ -975,9 +1003,11 @@ int docombo(const char *name, int zc){
 
 
 /* perform wicked command */
-int dowicked(const char *name, int rnum, int zc){
+int dowicked(const char *name, int rnum, int cb, int zc){
   VILLA *villa;
-  int i, j, omode, len, err, ksiz, vsiz, tran;
+  CBMAP *map;
+  int i, j, omode, len, err, ksiz, vsiz, tran, mksiz, mvsiz, rsiz;
+  const char *mkbuf, *mvbuf;
   char buf[32], *kbuf, *vbuf;
   CBLIST *list;
   printfflush("<Wicked Writing Test>\n  name=%s  rnum=%d\n\n", name, rnum);
@@ -990,22 +1020,32 @@ int dowicked(const char *name, int rnum, int zc){
   err = FALSE;
   tran = FALSE;
   vlsettuning(villa, 5, 10, 64, 64);
+  map = NULL;
+  if(cb) map = cbmapopen();
   for(i = 1; i <= rnum; i++){
     len = sprintf(buf, "%08d", myrand() % rnum + 1);
-    switch(myrand() % 16){
+    switch(cb ? (myrand() % 5) : myrand() % 16){
     case 0:
       putchar('O');
       if(!vlput(villa, buf, len, buf, len, VL_DOVER)) err = TRUE;
+      if(map) cbmapput(map, buf, len, buf, len, TRUE);
       break;
     case 1:
       putchar('K');
       if(!vlput(villa, buf, len, buf, len, VL_DKEEP) && dpecode != DP_EKEEP) err = TRUE;
+      if(map) cbmapput(map, buf, len, buf, len, FALSE);
       break;
     case 2:
-      putchar('D');
-      if(!vlout(villa, buf, len) && dpecode != DP_ENOITEM) err = TRUE;
+      putchar('C');
+      if(!vlput(villa, buf, len, buf, len, VL_DCAT)) err = TRUE;
+      if(map) cbmapputcat(map, buf, len, buf, len);
       break;
     case 3:
+      putchar('D');
+      if(!vlout(villa, buf, len) && dpecode != DP_ENOITEM) err = TRUE;
+      if(map) cbmapout(map, buf, len);
+      break;
+    case 4:
       putchar('G');
       if((vbuf = vlget(villa, buf, len, NULL)) != NULL){
         free(vbuf);
@@ -1013,11 +1053,12 @@ int dowicked(const char *name, int rnum, int zc){
         err = TRUE;
       }
       break;
-    case 4:
+    case 5:
       putchar('V');
+      if(vlvsiz(villa, buf, len) < 0 && dpecode != DP_ENOITEM) err = TRUE;
       if(!vlvnum(villa, buf, len) && dpecode != DP_ENOITEM) err = TRUE;
       break;
-    case 5:
+    case 6:
       putchar('X');
       list = cblistopen();
       cblistpush(list, buf, len);
@@ -1025,11 +1066,11 @@ int dowicked(const char *name, int rnum, int zc){
       if(!vlputlist(villa, buf, len, list)) err = TRUE;
       cblistclose(list);
       break;
-    case 6:
+    case 7:
       putchar('Y');
       if(!vloutlist(villa, buf, len) && dpecode != DP_ENOITEM) err = TRUE;
       break;
-    case 7:
+    case 8:
       putchar('Z');
       if((list = vlgetlist(villa, buf, len)) != NULL){
         cblistclose(list);
@@ -1042,11 +1083,18 @@ int dowicked(const char *name, int rnum, int zc){
         err = TRUE;
       }
       break;
-    case 8:
+    case 9:
       putchar('Q');
       if(vlcurjump(villa, buf, len, VL_JFORWARD)){
         for(j = 0; j < 3 && (kbuf = vlcurkey(villa, &ksiz)); j++){
           if(VL_CMPLEX(buf, len, kbuf, ksiz) > 0) err = TRUE;
+          if(strcmp(vlcurkeycache(villa, NULL), kbuf)) err = TRUE;
+          if((vbuf = vlcurval(villa, &vsiz)) != NULL){
+            if(strcmp(vlcurvalcache(villa, NULL), vbuf)) err = TRUE;
+            free(vbuf);
+          } else {
+            err = TRUE;
+          }
           free(kbuf);
           if(!vlcurnext(villa) && dpecode != DP_ENOITEM) err = TRUE;
         }
@@ -1054,11 +1102,18 @@ int dowicked(const char *name, int rnum, int zc){
         if(dpecode != DP_ENOITEM) err = TRUE;
       }
       break;
-    case 9:
+    case 10:
       putchar('W');
       if(vlcurjump(villa, buf, len, VL_JBACKWARD)){
         for(j = 0; j < 3 && (kbuf = vlcurkey(villa, &ksiz)); j++){
           if(VL_CMPLEX(buf, len, kbuf, ksiz) < 0) err = TRUE;
+          if(strcmp(vlcurkeycache(villa, NULL), kbuf)) err = TRUE;
+          if((vbuf = vlcurval(villa, &vsiz)) != NULL){
+            if(strcmp(vlcurvalcache(villa, NULL), vbuf)) err = TRUE;
+            free(vbuf);
+          } else {
+            err = TRUE;
+          }
           free(kbuf);
           if(!vlcurprev(villa) && dpecode != DP_ENOITEM) err = TRUE;
         }
@@ -1066,7 +1121,7 @@ int dowicked(const char *name, int rnum, int zc){
         if(dpecode != DP_ENOITEM) err = TRUE;
       }
       break;
-    case 10:
+    case 11:
       if(myrand() % 32 != 0){
         putchar('N');
         break;
@@ -1115,7 +1170,7 @@ int dowicked(const char *name, int rnum, int zc){
     kbuf = NULL;
     vbuf = NULL;
     if(!(kbuf = vlcurkey(villa, &ksiz)) || !(vbuf = vlcurval(villa, &vsiz)) ||
-       ksiz != 8 || vsiz != 8 || vlvnum(villa, kbuf, ksiz) < 1){
+       ksiz != 8 || vsiz % 8 != 0 || vlvnum(villa, kbuf, ksiz) < 1){
       pdperror(name);
       free(kbuf);
       free(vbuf);
@@ -1143,7 +1198,7 @@ int dowicked(const char *name, int rnum, int zc){
     kbuf = NULL;
     vbuf = NULL;
     if(!(kbuf = vlcurkey(villa, &ksiz)) || !(vbuf = vlcurval(villa, &vsiz)) ||
-       ksiz != 8 || vsiz != 8 || vlvnum(villa, kbuf, ksiz) < 1){
+       ksiz != 8 || vsiz % 8 != 0 || vlvnum(villa, kbuf, ksiz) < 1){
       pdperror(name);
       free(kbuf);
       free(vbuf);
@@ -1162,6 +1217,27 @@ int dowicked(const char *name, int rnum, int zc){
     pdperror(name);
     err = TRUE;
   }
+  if(map){
+    printfflush("Matching records ... ");
+    cbmapiterinit(map);
+    while((mkbuf = cbmapiternext(map, &mksiz)) != NULL){
+      mvbuf = cbmapget(map, mkbuf, mksiz, &mvsiz);
+      if(!(vbuf = vlget(villa, mkbuf, mksiz, &rsiz))){
+        pdperror(name);
+        err = TRUE;
+        break;
+      }
+      if(rsiz != mvsiz || memcmp(vbuf, mvbuf, rsiz)){
+        fprintf(stderr, "%s: %s: unmatched record\n", progname, name);
+        free(vbuf);
+        err = TRUE;
+        break;
+      }
+      free(vbuf);
+    }
+    cbmapclose(map);
+    if(!err) printfflush("ok\n");
+  }
   if(!vlclose(villa)){
     pdperror(name);
     return 1;
